# coding: utf-8

from os.path import dirname, join

import pysinsy

DATA_DIR = dirname(__file__)


def test_pysinsy_frontend():
    # http://sinsy.sp.nitech.ac.jp/sample/song070_f00001_063.xml

    xml_path = join(DATA_DIR, "song070_f00001_063.xml")

    sinsy = pysinsy.sinsy.Sinsy()
    assert sinsy.setLanguages("j", pysinsy.get_default_dic_dir())
    assert sinsy.loadScoreFromMusicXML(xml_path)

    for is_mono in [True, False]:
        label = sinsy.createLabelData(is_mono, 1, 1)
        for line in label.getData():
            print(line)

    sinsy.clearScore()


def test_pysinsy_synthesize():
    xml_path = join(DATA_DIR, "song070_f00001_063.xml")

    sinsy = pysinsy.sinsy.Sinsy()
    assert sinsy.setLanguages("j", pysinsy.get_default_dic_dir())
    assert sinsy.loadScoreFromMusicXML(xml_path)
    assert sinsy.loadVoices(pysinsy.get_default_htsvoice())

    wav, sr = sinsy.synthesize()
    assert len(wav) > 0 and wav.max() > 0
    assert sr == 48000

    sinsy.clearScore()


def test_pysinsy_highlevel():
    xml_path = join(DATA_DIR, "song070_f00001_063.xml")

    for _ in range(3):
        wav, sr = pysinsy.synthesize(xml_path)
        assert len(wav) > 0 and wav.max() > 0
        assert sr == 48000



# distutils: language = c++

from libcpp cimport bool
from libcpp.string cimport string
from libcpp.vector import vector

cdef extern from "LabelStrings.h" namespace "sinsy":
    cdef cppclass LabelStrings:
      void LabelStrings()
      size_t size()
      const char* const* getData()


import pkg_resources

from .sinsy import Sinsy
from .version import __version__  # noqa

DEFAULT_DIC_DIR = pkg_resources.resource_filename(__name__, "_dic")

DEFAULT_HTS_VOICE = pkg_resources.resource_filename(
    __name__, "htsvoice/nitech_jp_song070_f001.htsvoice"
)

# Global instance of sinsy
_global_sinsy = None


def get_default_dic_dir():
    """Returns default dictionary directory

    Returns:
        str: dictionary path
    """
    return DEFAULT_DIC_DIR


def get_default_htsvoice():
    """Returns default hts voice

    Returns:
        str: hts voice path
    """
    return DEFAULT_HTS_VOICE


def _lazy_init():
    global _global_sinsy
    _global_sinsy = Sinsy()
    _global_sinsy.setLanguages("j", get_default_dic_dir())
    _global_sinsy.loadVoices(get_default_htsvoice())


def synthesize(musicxml_path: str):
    """Synthesize waveform given muxic XML file

    Args:
        str: music XML path

    Returns:
        tuple: tuple of wav array of type float64 and sampling frequency

    Raises:
        RuntimeError: if fail to synthesize
    """
    global _global_sinsy
    if _global_sinsy is None:
        _lazy_init()

    _global_sinsy.loadScoreFromMusicXML(musicxml_path)
    wav, sr = _global_sinsy.synthesize()
    _global_sinsy.clearScore()

    return wav, sr
